@@ -18,3 +18,9 @@ pub mod auth;
 
 /// Storage handler.
 pub mod paste;
+
+/// Media type handler.
+pub mod mime;
+
+/// Helper functions.
+pub mod util;
