@@ -1,7 +1,9 @@
 use crate::auth;
 use crate::config::Config;
-use crate::header::ContentDisposition;
+use crate::header::{self, ContentDisposition};
+use crate::mime;
 use crate::paste::{Paste, PasteType};
+use crate::util;
 use actix_files::NamedFile;
 use actix_multipart::Multipart;
 use actix_web::{error, get, post, web, Error, HttpRequest, HttpResponse, Responder};
@@ -26,20 +28,45 @@ async fn serve(
     file: web::Path<String>,
     config: web::Data<Config>,
 ) -> Result<HttpResponse, Error> {
-    let mut path = config.server.upload_path.join(&*file);
+    let path = config.server.upload_path.join(&*file);
+    let mut path = util::glob_match_file(path)?;
     let mut paste_type = PasteType::File;
-    for (type_, alt_path) in &[(PasteType::Url, "url")] {
-        if !path.exists() || path.file_name().map(|v| v.to_str()).flatten() == Some(alt_path) {
-            path = config.server.upload_path.join(alt_path).join(&*file);
-            paste_type = *type_;
-            break;
+    if !path.exists() || path.is_dir() {
+        for type_ in &[PasteType::Url, PasteType::Oneshot] {
+            let alt_path = type_.get_path(&config.server.upload_path).join(&*file);
+            let alt_path = util::glob_match_file(alt_path)?;
+            if alt_path.exists()
+                || path.file_name().map(|v| v.to_str()).flatten() == Some(&type_.get_dir())
+            {
+                path = alt_path;
+                paste_type = *type_;
+                break;
+            }
         }
     }
     match paste_type {
-        PasteType::File => Ok(NamedFile::open(&path)?
-            .disable_content_disposition()
-            .prefer_utf8(true)
-            .into_response(&request)?),
+        PasteType::File | PasteType::Oneshot => {
+            let response = NamedFile::open(&path)
+                .map_err(|_| error::ErrorNotFound("file is not found or expired :("))?
+                .disable_content_disposition()
+                .set_content_type(
+                    mime::get_mime_type(&config.paste.mime_override, file.to_string())
+                        .map_err(error::ErrorInternalServerError)?,
+                )
+                .prefer_utf8(true)
+                .into_response(&request)?;
+            if paste_type.is_oneshot() {
+                fs::rename(
+                    &path,
+                    path.with_file_name(format!(
+                        "{}.{}",
+                        file,
+                        util::get_system_time()?.as_millis()
+                    )),
+                )?;
+            }
+            Ok(response)
+        }
         PasteType::Url => Ok(HttpResponse::Found()
             .header("Location", fs::read_to_string(&path)?)
             .finish()),
@@ -56,6 +83,7 @@ async fn upload(
     let connection = request.connection_info();
     let host = connection.remote_addr().unwrap_or("unknown host");
     auth::check(host, request.headers(), env::var("AUTH_TOKEN").ok())?;
+    let expiry_date = header::parse_expiry_date(request.headers())?;
     let mut urls: Vec<String> = Vec::new();
     while let Some(item) = payload.next().await {
         let mut field = item?;
@@ -79,8 +107,10 @@ async fn upload(
                 type_: paste_type,
             };
             let file_name = match paste_type {
-                PasteType::File => paste.store_file(content.get_file_name()?, &config)?,
-                PasteType::Url => paste.store_url(&config)?,
+                PasteType::File | PasteType::Oneshot => {
+                    paste.store_file(content.get_file_name()?, expiry_date, &config)?
+                }
+                PasteType::Url => paste.store_url(expiry_date, &config)?,
             };
             log::info!("{} ({}) is uploaded from {}", file_name, bytes_unit, host);
             urls.push(format!(
